#!/bin/bash
#################################################
# выбор способа разбиения диска
# 0 - разбивка задана файлом
# 1 - разбивка по-умолчанию ( = файлом)
# 2 - разбивка в диалоге
#################################################
declare temp_target_dev
declare GUID=0
declare -a part_n
declare -a part_name
declare -a part_start
declare -a part_end
declare -a part_type
declare -a part_size
declare -a part_fs_type
declare -a part_mountpoint
declare -a part_erase
dialog_fs_list='ext4 ext4 ext3 ext3 ext2 ext2 swap swap xfs xfs'

unset_globals()
{
    unset part_n
    unset part_name
    unset part_start
    unset part_end
    unset part_type
    unset part_size
    unset part_fs_type
    unset part_mountpoint
    unset part_erase
}

extended_part_id()
{
    if [ $GUID -eq 1 ]; then
        return
    fi
    for ((pnum=0; pnum<${#part_name[@]}; pnum=pnum+1)); do
        if [ "${part_type[$pnum]}" = "extended" ]; then
            echo $pnum
            return
        fi
    done
}

is_part_logical()
{
    index="$1"
    ext_id="$(extended_part_id)"

    if [ -n "$ext_id" ] && \
       [ ${part_start[$index]} -gt ${part_start[$ext_id]} ] && \
       [ ${part_start[$index]} -lt ${part_end[$ext_id]} ]; then
        return 0
    fi
    return 1
}

######################################################
# return 0 - точки монтирования root и boot существуют
#
# return 1 - совпали точки монтирования разделов
### return 1 - нет раздела /boot или / - устаревшее
# return 1 - нет раздела /
# return 1 - boot НЕ в формате в ext2/ext3
# если только корень, должен быть ext2/ext3
######################################################
check_root_boot()
{
    is_root="0"
    is_boot="0"
    boot_id=""
    root_id=""

    if [ -n "$(echo ${part_mountpoint[@]} | tr ' ' '\n' | uniq -d)" ]; then
        msg="Совпадают точки монтирования разделов"
        unset dialog_opt dialog_extra dialog_init
        dialog_call --msgbox "$msg"
        return 1
    fi

    for ((pnum=0; pnum < ${#part_size[*]}; pnum=pnum+1)); do
        if echo "${part_mountpoint[$pnum]}" | grep -E -q '^/{1,}$'; then
            is_root="1"
            root_id=$pnum
        elif echo "${part_mountpoint[$pnum]}" | grep -E -q '^/boot/?$'; then
            is_boot="1"
            boot_id=$pnum
        fi
    done

    if [ $is_root -eq 0 ]   # Нет рута
    then
        msg="Раздел / обязательный"
        unset dialog_opt dialog_extra dialog_init
        dialog_call --msgbox "$msg"
        return 1
    fi

    # Проверяем тип ФС
    # Есть рут и нет бута
    if [ $is_root -eq 1 -a $is_boot -eq 0 ]
    then
        if echo "${part_fs_type[$root_id]}" | grep -qv "^ext[2-3]$"
        then
            msg="Загрузочный раздел (/) должен быть в формате ext2/ext3"
            unset dialog_opt dialog_extra dialog_init
            dialog_call --msgbox "$msg"
            return 1
        fi
    # Есть рут и есть бут
    elif [ $is_root -eq 1 -a $is_boot -eq 1 ]
    then
        if echo "${part_fs_type[$boot_id]}" | grep -qv "^ext[2-3]$" 
        then
            msg="Загрузочный раздел (/boot) должен быть в формате ext2/ext3"
            unset dialog_opt dialog_extra dialog_init
            dialog_call --msgbox "$msg"
            return 1
        fi
    fi

    # рут меньше 20Гб?
    if [ "$((${part_end[$root_id]}-${part_start[$root_id]}))" -lt "$(human_bytes_to_sectors "20G")" ]
    then
        msg="Предупреждение: корневой раздел менее 20 Гб! (${part_size[$root_id]})\nПолный набор пакетов может не поместиться.\nПродолжить?"
        if [[ $dialog_min != 1 ]]
        then
            unset dialog_opt dialog_extra dialog_init
            dialog_call --yesno "$msg"
            [[ $? != 0 ]]&& return 2    # Назад
        else
            dialog_infobox "$msg"
        fi
    fi

    return 0    # Все ОК!
}

create_part_list()
{
    [[ "$target_type" != "device" ]] && return 0

    [[ -z "$TARGET_DEV" ]] && return 33

    TITLE="Разметка диска"
    local return_code=

    # create a copy of TARGET_DEV's partition table
    # временный файл
    temp_target_dev=$INST_LOCAL/part_edit/temp_target_dev/$TARGET_DEV

    # продолжаем работу с текущей разбивкой
    if [ -e "$temp_target_dev" ]; then
        part_resume_flg=1
    # иначе считываем параметры
    else
        part_resume_flg=0
        FDISK_TEMP=$INST_LOCAL/part_edit/fdisk_temp
        total_sectors=$(cat /sys/block/${TARGET_DEV#/dev/}/size)
        sector_size=$(cat /sys/block/${TARGET_DEV#/dev/}/queue/logical_block_size)
        align=$((1024*1024/${sector_size}))

        # clear
        rm -rf $INST_LOCAL/part_edit/temp_target_dev/$(dirname $TARGET_DEV)

        mkdir -p $INST_LOCAL/part_edit/temp_target_dev/$(dirname $TARGET_DEV)
        truncate -s $(LC_ALL=C fdisk -l $TARGET_DEV | grep "Disk" | cut -d' ' -f5) $temp_target_dev
        sfdisk -d $TARGET_DEV 2>>$INST_LOG | sfdisk $temp_target_dev >> $INST_LOG 2>&1
        disklabel=$(LC_ALL=C fdisk -l $temp_target_dev | grep Disklabel | cut -d' ' -f3)

        if [ $total_sectors -gt 4294967296 ]; then
            if [ $dialog_min == 0 ]; then
                msg="Выбран тип таблицы разделов GPT."
                dialog_call --msgbox "$msg"
                GUID=1
            elif [ -z "$disklabel" ]; then
                GUID=1
            elif [ "$disklabel" != "gpt" ]; then
                unset dialog_opt dialog_extra dialog_init
                dialog_call --yesno "Для выбранного диска нужно использовать структуру разделов GPT,
однако, диск уже размечен в формате $disklabel. Если вы продолжите,
разделы на диске будут уничтожены. Продолжить?"
                ret=$?
                # Да
                if [ $ret -eq 0 ]; then
                    GUID=1
                # Нет
                elif [ $ret -eq 1 ]; then
                    do_fatal "Диск содержит некорректную таблицу разделов"
                fi
            elif [ "$disklabel" = "gpt" ]; then
                GUID=1
            fi
        else
            if [ $dialog_min != 0 ]; then
                msg="\n      Выбран тип таблицы разделов MBR."
                dialog_infobox "$msg"
                GUID=0
            elif [ "$disklabel" = "gpt" ]; then
                unset dialog_opt dialog_extra dialog_init
                dialog_call --yesno "Диск уже размечен в формате $disklabel. \nРазметить диск в формате MBR?"
                if [ $? -eq 0 ]; then
                    GUID=0
                else
                    GUID=1
                fi
            elif [ "$disklabel" = "dos" ]; then
                GUID=0
            elif [ -z "$disklabel" ]; then
                GUID=0
            else
                unset dialog_opt dialog_extra dialog_init
                dialog_call --yesno "Диск содержит неизвестную таблицу разделов ($disklabel).  \
    Разметить диск в формате MBR?"
                if [ $? -eq 0 ]; then
                    GUID=0
                else
                    do_fatal "Неизвестная таблица разделов ($disklabel)."
                fi
            fi
        fi
    fi


    case $format_mode in
    # Задан файл разбивки
    0|1)
        (if [ $GUID -eq 1 ]; then
            echo "g"
        else
            echo "o"
        fi
        echo "w") | $FDISK $temp_target_dev >> $INST_LOG 2>&1

        inst_cfg_define
        return_code=$?
        ;;
    # Разбивка вручную
    2)
        dev_partition_dialog
        return_code=$?
        ;;
    esac

    if [[ $return_code == 33 ]]; then   # Назад
        TARGET_DEV=
        PART_PREFIX=
        CONF_FILE=
        return $return_code
    else
        return $return_code
    fi
}

delete_free_space()
{
declare -a local tmp_part_n=
declare -a local tmp_part_name=
declare -a local tmp_part_start=
declare -a local tmp_part_end=
declare -a local tmp_part_type=
declare -a local tmp_part_size=
declare -a local tmp_part_fs_type=
declare -a local tmp_part_mountpoint=
declare -a local tmp_part_erase=

    tmp_pnum=0;
    for ((pnum=0; pnum<${#part_name[@]}; pnum=pnum+1)); do
       if echo ${part_name[$pnum]} | grep -q "<free space>"; then
            continue
       fi
            tmp_part_n[$tmp_pnum]=${part_n[$pnum]}
            tmp_part_name[$tmp_pnum]=${part_name[$pnum]}
            tmp_part_start[$tmp_pnum]=${part_start[$pnum]}
            tmp_part_end[$tmp_pnum]=${part_end[$pnum]}
            tmp_part_type[$tmp_pnum]=${part_type[$pnum]}
            tmp_part_size[$tmp_pnum]=${part_size[$pnum]}
            tmp_part_fs_type[$tmp_pnum]=${part_fs_type[$pnum]}
            tmp_part_mountpoint[$tmp_pnum]=${part_mountpoint[$pnum]}
            tmp_part_erase[$tmp_pnum]=${part_erase[$pnum]}
            tmp_pnum=$(($tmp_pnum+1))
    done
    part_n=("${tmp_part_n[@]}")
    part_name=("${tmp_part_name[@]}")
    part_start=("${tmp_part_start[@]}")
    part_end=("${tmp_part_end[@]}")
    part_type=("${tmp_part_type[@]}")
    part_size=("${tmp_part_size[@]}")
    part_fs_type=("${tmp_part_fs_type[@]}")
    part_mountpoint=("${tmp_part_mountpoint[@]}")
    part_erase=("${tmp_part_erase[@]}")
}

select_format_variant()
{
    # format_mode
    # 0 - пользовательская/сетевая разбивка из файла
    # 1 - разбивка из файла по умолчанию
    # 2 - ручная разбивка

    [[ "$target_type" != "device" ]] && return 0

    TITLE="Разметка диска"

    if [ -n "$CONF_FILE" ] && [[ "$CONF_FILE" == "$CONF_FILE_DEFAULT" ]] ; then
        format_mode=1;
    elif [ -n "$CONF_FILE" ]; then
        format_mode=0;
        return 0;
    elif [ "$network_flg" -eq 1 ] &&  [ -f "${REMOTE_LOG_DIR}/conf/install.cfg" ]; then
        format_mode=0;
        CONF_FILE="${REMOTE_LOG_DIR}/conf/install.cfg"
        msg="Задан конфигурационный файл $CONF_FILE"
        unset dialog_opt dialog_extra dialog_init
        dialog_call --msgbox "$msg"
        return 0;
    else
        if [ ! -f "$CONF_FILE_DEFAULT" ]; then
            msg="Ошибка: отсутствует конфигурационный файл по умолчанию $CONF_FILE_DEFAULT"
            unset dialog_opt dialog_extra dialog_init
            dialog_call --msgbox "$msg"
            do_fatal
        fi
        format_mode=1
        CONF_FILE="$CONF_FILE_DEFAULT"
    fi

    [ "$dialog_min" -eq 1 ] && return 0;

    local val=""
    while [ -z "$val" ]; do
        unset dialog_opt dialog_extra dialog_init
        dialog_opt+=("1" "Разбиение диска на разделы по умолчанию" "off")
        dialog_opt+=("2" "Разбиение диска вручную"                 "off")
        val=$(dialog_call --radiolist "Выберите вариант разбиения диска:" "Выбрать" "Назад")
        ret=$?
        if [[ $ret == 1 ]]; then                # Назад
            return 33
        elif [[ $ret != 0 ]]; then
            return $ret
        fi
    done

    format_mode="$val"

    if [ $format_mode -eq 1 ]; then
        msg="Задана разбивка по-умолчанию"
        [ $dialog_min -eq 1 ] && dialog_infobox "$msg" || echo "$msg" >> $INST_LOG
    elif [ $format_mode -eq 2 ]; then
        msg="Разбивка будет произведена вручную"
        [ $dialog_min -eq 1 ] && dialog_infobox "$msg" || echo "$msg" >> $INST_LOG
    fi

    return 0
}

# Установка по пользовательской конфигурации
inst_cfg_define()
{
    local ret=
    is_whole_space=0    # Есть ли раздел размера "все оставшееся пространство". 0 - нет, 1 - есть
    whole_space_num=    # Номер  раздела размера "все оставшееся пространство"
    sum_size=0          # Суммарный размер заданных разделов

    # 145751 Без очистки получаем чудо разметку
    unset_globals

    part_count=`cat $CONF_FILE | grep -v "^#"| sed '/^$/d' | grep -cv '=' 2>>$INST_LOG`
    if [[ $? != 0 ]] || [ -z "$part_count" ] || [ $part_count -lt 1 ]
    then
        msg="\n  Количество разделов должно быть не менее одного"
        dialog_msgbox "$msg" 7 50
        return 1
    fi

    # Присвоение с 1-го элемента
    while read line; do
        if [ $GUID -eq 0 ]; then
            if [ ${#part_size[@]} -lt 3 ]; then
                part_type+=("primary")
            elif [ ${#part_size[@]} -eq 3 ]; then
                part_size+=("")
                part_fs_type+=("")
                part_mountpoint+=("")
                part_erase+=("n")
                part_type+=("extended")
            else
                part_type+=("logical")
            fi
        fi
        part_size+=(`echo $line | cut -d' ' -f1 2>>$INST_LOG` )
        part_fs_type+=(`echo $line | cut -d' ' -f2 2>>$INST_LOG` )
        part_mountpoint+=(`echo $line | cut -d' ' -f3 2>>$INST_LOG` ) 
        part_erase+=("y")
    done < <(grep -v "^#" $CONF_FILE | sed "/^$/d" | sed "s/[[:space:]]\+/ /g" | grep -v "=")

    # проверка входных параметров
    for ((pnum=0; pnum < ${#part_size[@]}; pnum = pnum+1))
    do
        [ $GUID -eq 0 ] && [ "${part_type[$pnum]}" = "extended" ] && continue
        # Проверка на заполнение
        [ -z "${part_size[$pnum]}" ] && msg="\n          Пустой параметр part_size" && dialog_msgbox "$msg" 7 50 && return 1
        [ -z "${part_fs_type[$pnum]}" ]         && msg="\n          Пустой параметр part_fs_type" && dialog_msgbox "$msg" 7 50 && return 1
        [ "${part_fs_type[$pnum]}" != "swap"  ] && [ -z "${part_mountpoint[$pnum]}" ]      && msg="\n          Пустой параметр part_mountpoint" && dialog_msgbox "$msg" 7 50 && return 1

        #проверка на корректность
        if ! echo ${part_fs_type[$pnum]} | grep -q -x -e 'ext[2-4]' -e 'swap' -e 'xfs'; then
            msg="Некорректный параметр part_fs_type[$pnum]: ${part_fs_type[$pnum]}" 
            unset dialog_opt dialog_extra dialog_init
            dialog_call --msgbox "$msg"
            return 1
        fi

        if [ "${part_fs_type[$pnum]}" != "swap"  ] && ! echo ${part_mountpoint[$pnum]} | grep -E -q -x '/[[:alnum:]]*/?'; then
            msg="Некорректный параметр part_mountpoint[$pnum]: ${part_mountpoint[$pnum]}"
            dialog_call --msgbox "$msg"
            unset dialog_opt dialog_extra dialog_init
            return 1
        fi  

        if ! echo ${part_size[$pnum]} | grep -E -q -o '^[0-9]*[M,G,T]?'; then
            msg="Некорректный параметр part_size[$pnum]: ${part_size[$pnum]}"
            unset dialog_opt dialog_extra dialog_init
            dialog_call --msgbox "$msg"
            return 1
        fi

        # конвертация в сектора
        part_size[$pnum]=$(human_bytes_to_sectors "${part_size[$pnum]}")

        if [ ${part_size[$pnum]} -gt 0 ]
        then
            # суммарный размер в секторах
            let sum_size+=${part_size[$pnum]}

        # Проверка на "все оставшееся пространство", задается единожды
        elif [ ${part_size[$pnum]} -eq 0 -a $is_whole_space -eq 0 ]
        then
            is_whole_space=1
            whole_space_num=$pnum
        elif [ ${part_size[$pnum]} -eq 0 -a "$is_whole_space" -eq 1 ]
        then
            msg="Ошибка: раздел \"все оставшееся пространство\" уже задан!"
            unset dialog_opt dialog_extra dialog_init
            dialog_call --msgbox "$msg"
            return 1
        fi
    done

    if [[ $is_whole_space == 1 ]]
    then
        # Вычисление размера раздела "все оставшееся пространство" в секторах
        if [ $GUID -eq 0 ] && [ ${#part_size[@]} -gt 3 ]; then
            part_size[$whole_space_num]=$[$total_sectors-(4*${align}+$sum_size)]
        else
            part_size[$whole_space_num]=$[$total_sectors-(2*${align}+$sum_size)]
        fi
        let sum_size+=${part_size[$whole_space_num]}
    fi

    # Проверка суммарного размера разделов
    if [ $total_sectors -lt $sum_size ]
    then
        msg="Ошибка: суммарный размер разделов превышает размер целевого устройства"
        unset dialog_opt dialog_extra dialog_init
        dialog_call --msgbox "$msg"
        # сброс CONF_FILE для выбора типа разбивки вручную
#       format_mode=2
        CONF_FILE=
        return 2
    fi

    if [ $GUID -eq 1 ] ; then
        # выделяем место под secondary gpt header
        last_part_id=$((${#part_size[@]}-1))
        part_size[$last_part_id]=$((${part_size[$last_part_id]}-33))
    fi

    echo "fdisk input is:" >> $INST_LOG
    (
    for ((pnum=0; pnum < ${#part_size[@]}; pnum = pnum+1)); do
        echo "n"
        if [ $GUID -eq 1 ]; then
            echo ""
            echo ""
            echo "+$((${part_size[$pnum]}-1))"
        elif [ $pnum -lt 3 ]; then
            echo ""
            echo ""
            echo ""
            echo "+$((${part_size[$pnum]}-1))"
        elif [ $pnum -eq 3 ]; then
            echo ""
            echo ""
            echo ""
            part_size[$(($pnum+1))]=$((${part_size[$(($pnum+1))]}-2048))
        fi
        # дообавление логического раздела
        if [ $GUID -eq 0 ] && [ $pnum -gt 3 ]; then
            echo ""
            echo "+$((${part_size[$pnum]}-1))"
        fi
        if [ "${part_fs_type[$pnum]}" = "swap" ]; then
            echo "t"
            echo ""
            echo "82"
        fi
    done; echo "w"
    ) | tee -a $INST_LOG | $FDISK $temp_target_dev >> $INST_LOG 2>&1
    ret=$?
    # fdisk при неудачной разбивке (типы выход размера за пределы) возвращает 0
    if [[ $ret == 1 ]]; then
        msg="Ошибка fdisk"
        unset dialog_opt dialog_extra
        dialog_call --msgbox "$msg"
        return 2
    fi

    read_sfdisk_in $temp_target_dev

    # Проверка на наличие / и /boot
    check_root_boot
    ret=$?
    [[ $ret != 0 ]] && return $ret   # Проверка не пройдена

    return 0
}

# Вывод информации о разделах
show_disk_part()
{
    [[ "$target_type" != "device" ]] && return 0

    if [[ $dialog_min == 0 ]]
    then
        unset dialog_opt dialog_extra dialog_init
        echo -n " --no-collapse --backtitle \"$BACKTITLE\" --title \"$TITLE\" --output-fd 5 \
            --yes-label \"Продолжить\" --no-label \"Назад\" --yesno \""  > $DIALOG_FILE
        [[ $GUID == 0 ]] && \
        echo -e "Таблица разделов DOS\n"                    >> $DIALOG_FILE
        [[ $GUID == 1 ]] && \
        echo -e "Таблица разделов GPT\n"                    >> $DIALOG_FILE
        echo -e "Разбиение диска на разделы "               >> $DIALOG_FILE
        echo -e  "будет произведено следующим образом:\n"   >> $DIALOG_FILE

        (echo "Устройство Размер ФС Путь Форматировать"
        for ((pnum=0; pnum < ${#part_name[@]}; pnum=pnum+1)); do
                echo -n "${TARGET_DEV}${part_n[$pnum]} "
                echo -n "${part_size[$pnum]} "
                [ -z "${part_fs_type[$pnum]}" ]        && echo -n "- " || echo -n "${part_fs_type[$pnum]} "
                if [ "${part_fs_type[$pnum]}" != "swap" ] && [ "${part_type[$pnum]}" != "extended" ]; then
                    [ -z "${part_mountpoint[$pnum]}" ] && echo -n "- " || echo -n "${part_mountpoint[$pnum]} "
                    [ -z "${part_erase[$pnum]}" ]      && echo -n "n"  || echo -n "${part_erase[$pnum]}"
                fi
                echo ""
        done) | column -t >> $DIALOG_FILE

        echo -e "\nПродолжить разбиение диска?"     >> $DIALOG_FILE
        echo -e "ВНИМАНИЕ! Все данные на разделах, помеченных для форматирования будут уничтожены!\" $((`cat $DIALOG_FILE | wc -l`+6)) 65" >> $DIALOG_FILE

        $DIALOG $D_OPT --no-collapse --output-fd 5 --file $DIALOG_FILE 5> $tmp
        ret=$?
        if [[ $ret == 0 ]]    # Да
        then
            return 0
        elif [[ $ret == 1 ]]    # Назад
        then
            # При ручной разбивке возвращаемся к редактированию разделов
            if [[ "$format_mode" == 2 ]]
            then
                return 33
            else
                # Возврат на выбор целевого устройства
                # Сброс настроек на дефолтные
                CONF_FILE="$CONF_FILE_DEFAULT"
                TARGET_DEV=
                PART_PREFIX=
                format_mode=1
                return 33
            fi
        elif [[ $ret != 0 ]]; then
            return $ret
        fi
    else
        msg="`echo -n -e "\nЦелевое устройство ${TARGET_DEV}\nЗадана разбивка диска:\n"
        (echo -e "Раздел;ФС;Монтитрование;Размер"
        local var_i=0
        while [ $var_i -lt $part_count ]; do
            local part_size_cr=$(echo ${part_size[$var_i]}| sed 's/[[:alpha:]]//g;s/\..*$//g')
            if [ "$part_size_cr" -eq 0 -o -z "${part_size[$var_i]}" ]
            then
                echo "${TARGET_DEV}${PART_PREFIX}${part_n[$var_i]};${part_fs_type[$var_i]};${part_mountpoint[$var_i]};Оставшееся пространство"
            elif [ "${part_type[$var_i]}" == "extended" ]
            then
                echo "${TARGET_DEV}${PART_PREFIX}${part_n[$var_i]};${part_type[$var_i]};;${part_size[$var_i]}"
            elif [ "${part_fs_type[$var_i]}" == "cur" ]
            then
                echo "${TARGET_DEV}${PART_PREFIX}${part_n[$var_i]};${part_fs_type[$var_i]}(${part_fs_type_cur[$var_i]});${part_mountpoint[$var_i]};${part_size[$var_i]}"
            else
                echo "${TARGET_DEV}${PART_PREFIX}${part_n[$var_i]};${part_fs_type[$var_i]};${part_mountpoint[$var_i]};${part_size[$var_i]}"
            fi
            let var_i++
        done) | column -s ';' -t
        `"
        dialog_infobox "$msg"
        return 0
    fi
}


# Цифры без размерности = сектора
human_bytes_to_sectors()
{
    sector_size=$(LC_ALL=C fdisk -l $TARGET_DEV| grep Units | grep -o "[0-9]* bytes" | cut -d' ' -f1)

    # целая часть
    size=$(echo $1 | sed 's/[[:alpha:]]//g;s/\..*$//g')

    # дробная часть
    fraction=$(echo $1 | sed 's/[[:alpha:]]//g;/\./!s/.*//g;/\./s/.*\.//g')
    if [ -n "$fraction" ]
    then
        fraction_rank=$(echo -n "$fraction" | wc -m)
    else
        fraction=0
        fraction_rank=1
    fi

    prefix=$(echo $1 | sed 's/[[:digit:]]//g;s/\.//g')
    [ -z "$prefix" ] && echo $1 && return
    size_prefix[0]="b"
    size_prefix[1]="k"
    size_prefix[2]="M"
    size_prefix[3]="G"
    size_prefix[4]="T"
    for ((i=0; i<${#size_prefix[*]}; i=i+1)); do
        [ "${size_prefix[$i]}" = "$prefix" ] && break
    done
    if [ "$i" -gt $((${#size_prefix[*]}-1)) ]; then
        msg="Ошибка в функции human_bytes_to_sectors()"
        unset dialog_opt dialog_extra dialog_init
        dialog_call --msgbox "$msg"

        do_fatal
    fi
    while [ "$i" -gt 0 ]; do
        size=$((size * 1024 + fraction * 1024 / (fraction_rank * 10)))
        i=$(($i-1))
    done
    size=$(($size/$sector_size))
    echo $size
}

sectors_to_human_bytes()
{
    sector_size=$(LC_ALL=C fdisk -l $TARGET_DEV| grep Units | grep -o "[0-9]* bytes" | cut -d' ' -f1)
    size=$1
    size=$(($size*$sector_size))
    size_f=""
    size_prefix[0]="b"
    size_prefix[1]="k"
    size_prefix[2]="M"
    size_prefix[3]="G"
    size_prefix[4]="T"
    local i=0;
    while [ "$(($size/1024))" -gt 0 ] && [ $i -lt 5 ]; do
        size=$((100*$size/1024))
        size=$(echo $size | sed 's/..$/.&/')
        i=$((i+1))
        size_f=$(echo $size | cut -d'.' -f2)
        size=$(echo $size | cut -d'.' -f1)
    done
    echo "$size.$size_f${size_prefix[$i]}"
}

part_delete_dialog()
{
    local ret=
    index="$1"
    unset dialog_opt dialog_extra dialog_init
    dialog_extra+=("--help-button" "--help-label" "Восстановить")
    dialog_call --yesno "Раздел ${part_name[$index]} будет удалён! Продолжить?\n\nВосстановить - вернуть начальную разметку\nОчистить     - создать пустую таблицу разделов" "Да" "Очистить" "Отмена"
    ret=$?
    # Если удаляем пустое пространство
    if [ $ret -eq 0 ] && echo ${part_name[$index]} | grep -q "<"; then
        return 0
    elif [ $ret -eq 0 ]; then
        part_name[$index]=""
        part_start[$index]=""
        part_end[$index]=""
        part_type[$index]=""
        part_size[$index]=""
        part_fs_type[$index]=""
        part_mountpoint[$index]=""
        part_erase[$index]=""

        # записываем наши действия в fdisk_in
        true_parts_count=$(for ((i=0; i<${#part_name[@]}; i=i+1)); do echo ${part_name[$i]} | grep -v "<free space>"; done | wc -l)
        (echo "d"
         if [ $true_parts_count -gt 1 ]; then
            echo $(echo ${part_n[$index]} | grep -o "[0-9]*$")
         fi
         echo "w") | $FDISK $temp_target_dev >> $INST_LOG 2>&1
    elif [ $ret -eq 1 ]; then
        return 1
    elif [ $ret -eq 2 ]; then  # help/восстановить разметку
        sfdisk -d $TARGET_DEV 2>>$INST_LOG | sfdisk $temp_target_dev >> $INST_LOG 2>&1
        unset_globals
        read_sfdisk_in $temp_target_dev
    elif [ $ret -eq 3 ]; then  # extra/очистить разметку
        sfdisk --delete $temp_target_dev &>> $INST_LOG
        unset_globals
        read_sfdisk_in $temp_target_dev
    elif [[ $ret == $DIALOG_ESC ]]    # ESC button
    then
        do_exit
    fi
    return 0
}

part_edit_dialog()
{
    local default_item=
    index="$1";
    ext_id=$(extended_part_id)
    [ "$index" = "$ext_id" ] && return 0

    ### Part edit
    p_size=${part_size[$index]}
    p_end=${part_end[$index]}
    p_type=${part_type[$index]}
    p_erase=${part_erase[$index]}
    [ -z "$p_erase" ] && p_erase="n" && part_erase[$index]="n"
    p_fs_type="${part_fs_type[$index]}"
    p_mountpoint="${part_mountpoint[$index]}"

    # Предустановки для типов разделов
    # считаем primary разделы вместе с extended
    primary_parts_count=$(LC_ALL=C fdisk -l $temp_target_dev | sed -n "1,/Extended/p" | grep -c -e Linux$ -e Extended -e 'Linux swap')
    if [ $GUID -eq 0 ]; then
        if [ "$primary_parts_count" -eq 3 ] && \
            [ -z "$ext_id" ] && [ $index -eq 3 ]; then
            p_type="extended"
        elif echo "${part_name[$index]}" | grep -q "^ "; then
            p_type="logical"
        # проверка на создание раздела до extended при наличии 3-х primary
        elif [ "$primary_parts_count" -ge 4 ] && [ -n "$ext_id" ] && \
            echo "${part_name[$index]}" | grep -q "^<" && \
            [ "${part_start[$index]}" -lt "${part_start[$ext_id]}" ]; then
            msg="Первичных разделов + расширенный не может быть больше 4"
            unset dialog_opt dialog_extra dialog_init
            dialog_call --msgbox "$msg"
            return 0
        # проверка на наличие более 4-х primary разделов
        elif [ "$primary_parts_count" -gt 4 ] && [ -z "$ext_id" ] && \
            echo "${part_name[$index+1]}" | grep -q "^<"; then
            msg="Ошибка: первичных разделов не может быть больше 4"
            unset dialog_opt dialog_extra dialog_init
            dialog_call --msgbox "$msg"
            do_fatal
        else
            p_type="primary"
        fi
    fi


    ## part add
    while true; do
        ## пропуск, если выбран не <free space>
        if [ $GUID -eq 0 ] && \
        echo "${part_name[$index]}" | grep -q "^<" && \
        [ "$(fdisk -l $temp_target_dev | grep -c Linux$)" = "4" ] && [ -z "$ext_id" ]; then
            msg="\n            Первичных разделов не может быть больше 4"
            unset dialog_opt dialog_init
            dialog_call --msgbox "$msg"
            # словил зацикливание
            # continue
            break
        fi
        default_item=
        unset dialog_opt dialog_extra dialog_init
        if echo "${part_name[$index]}" | grep -q "<"; then
            dialog_opt+=("size"         "Размер              : $p_size")
            default_item="size"
        fi
        if [ $GUID -eq 0 ]; then
            $(is_part_logical $index) && p_type="logical" && part_type[$index]="$p_type"
            dialog_opt+=("type"         "Тип раздела         : $p_type")
            default_item="type"
        fi
        if [ $GUID -eq 1 ] || ([ "$p_type" != "extended" ] && [ -n "$p_type" ]); then
            if [ "$p_fs_type" != "swap" ]; then
                # Новый раздел форматируем
                if echo "${part_name[$index]}" | grep -q "^<"; then
                    p_erase="y"
                    dialog_opt+=("erase"       "Форматировать       : Да")
                else
                    dialog_opt+=("erase"       "Форматировать       : $([ $p_erase = "y" ] && echo Да || echo Нет)")
                    default_item="erase"
                fi
            fi
                dialog_opt+=("fs_type"      "Файловая система    : $p_fs_type")
                default_item="fs_type"
            if [ "$p_fs_type" != "swap" ] && [ -n "$p_fs_type" ]; then
                dialog_opt+=("mountpoint"   "Точка монтирования  : $p_mountpoint")
                default_item="mountpoint"
            fi
        fi
        dialog_extra=("--default-item" "$default_item")
        item=$(dialog_call --menu "Редактирование раздела на устройстве $TARGET_DEV:" "Выбрать" "Сохранить" "Назад")
        ret="$?"

        # Кнопка "Выбрать" - настройка раздела
        if [ "$ret" -eq 0 ]; then
            ## Part tune dialog
            case $item in
                size )
                            val=$(dialog_call --inputbox " Введите размер раздела (Доступно ${part_size[$index]})\n \
Формат ввода <Целое число><M/G/T> (Напр. 120G)\n \
0 - Использовать все доступное пространство" "Принять" "Назад")
                            [ "$?" -ne 0 ] && continue
                            val=$(echo $val | sed 's/[[:space:]]//g' )
                            if [ "$val" = "0" ] || [ -z "$val" ]; then
                                p_size="${part_size[$index]}"
                                p_end="${part_end[$index]}"
                                continue
                            elif echo "$val" | grep -q -v -E "^[0-9]*[[:space:]]?[M,G,T]$" ||
                                [ $(echo "$val" | grep -o "[0-9]*") -lt 0 ]; then
                                msg="Неправильно введен размер!"
                                unset dialog_opt dialog_extra dialog_init
                                dialog_call --msgbox "$msg"
                                continue;
                            else
                                size_in_sectors="$(human_bytes_to_sectors "$val")"
                                # Правки багов 146053, 144237
                                disklabel=$(LC_ALL=C fdisk -l $temp_target_dev | grep Disklabel | cut -d' ' -f3)
                                size_in_sectors="$(human_bytes_to_sectors "$val")"
                                limit_in_sectors="$(human_bytes_to_sectors "2T")"
                                if [ "$size_in_sectors" -ge "$limit_in_sectors" ] && [ "$disklabel" = "dos" ]; then
                                    unset dialog_opt dialog_extra
                                    dialog_call --yesno "При разметке $disklabel раздел не может составлять 2Тб и более. \nРазметить диск в формате GPT?"
                                    ret=$?
                                    # Да
                                    if [ "$ret" = "0" ]; then
                                        GUID=1
                                        msg="Диск теперь размечен в формате GPT."
                                        echo -e "w\nY\n" | gdisk $temp_target_dev >> $INST_LOG 2>&1 # меняем метку с сохранением данных
                                        dialog_call --msgbox "$msg"
                                        break
                                    # Нет
                                    else
                                        do_fatal "Возник конфликт меток"
                                    fi
                                fi

				if [ "$size_in_sectors" -gt "$((${part_end[$index]}-${part_start[$index]}))" ]; then
                                    msg="Нет столько места!"
                                    unset dialog_opt dialog_extra dialog_init
                                    dialog_call --msgbox "$msg"
                                    continue
                                fi
                                p_size="$val"
                                p_end="+$(($size_in_sectors-1))"
                            fi
                    ;;
                type )
                        if [ $GUID -eq 1 ]; then
                            continue
                        fi
                        unset dialog_opt dialog_extra dialog_init
                        if echo ${part_name[$index]} | grep -q "free"; then
                            if $(is_part_logical $index); then
                                dialog_opt+=("logical" "logical")
                            else
                                if [ "$p_type" = "extended" ]; then
                                    dialog_opt+=("extended" "extended")
                                    dialog_opt+=("primary" "primary")
                                else
                                    dialog_opt+=("primary" "primary")
                                    [ -z "$ext_id" ] &&
                                    dialog_opt+=("extended" "extended")
                                fi
                            fi
                        else
                           dialog_opt+=("${part_type[$index]}" "${part_type[$index]}")
                        fi
                        val=$(dialog_call --menu "Выберите тип раздела:" "Выбрать" "Назад")
                        [ "$?" -ne 0 ] && continue
                        p_type="$val"
                        part_type[$index]="$val"
                        if [ "$p_type" = "extended" ]; then
                            part_fs_type[$index]=""
                            part_mountpoint[$index]=""
                            part_erase[$index]="n"
                            p_erase="n"
                            p_fs_type=
                            p_mountpoint=
                        fi
                    ;;
                erase )
                        unset dialog_opt dialog_extra dialog_init
                        dialog_call --yesno "Форматировать радел?"
                        if [ $? -eq 0 ]; then
                            p_erase="y"
                        else
                            p_erase="n"
                            p_fs_type="$(LC_ALL=C lsblk -n -oFSTYPE ${TARGET_DEV}${part_n[$index]} 2>>$INST_LOG)"
                            p_mountpoint=""
                        fi
                    ;;
                fs_type )
                        part_fs_list="$dialog_fs_list"
                        unset dialog_opt dialog_extra dialog_init
                        if [ $p_erase = "n" ]; then
                            if [ -z "$p_fs_type" ]; then
                                dialog_opt+=("unknown" "unknown (current)")
                            else
                                dialog_opt+=("$p_fs_type" "$p_fs_type (current)")
                            fi
                        else
                            dialog_opt+=($part_fs_list)
                        fi
                        val=$(dialog_call --menu "Выберите тип файловой системы" "Выбрать" "Назад")
                        [ "$?" -ne 0 ] && continue
                        [ "$val" = "swap" ] && p_mountpoint=""
                        [ "$val" = "unknown" ] && p_mountpoint="" && continue
                        p_fs_type="$val"
                    ;;

                mountpoint )
                        unset dialog_opt dialog_extra
                        dialog_init="$p_mountpoint"
                        val=$(dialog_call --inputbox "Введите точку монтирования" "Принять" "Назад")
                        [ "$?" -ne 0 ] && continue
                        val=$(echo $val | sed 's/^[[:space:]]//g' | sed 's/[[:space:]]$//g')
                        if echo $val | grep -v -q "^/"; then
                            msg="Точка монтирования должна начинаться с '/'!"
                            unset dialog_opt dialog_extra dialog_init
                            dialog_call --msgbox "$msg"
                            continue;
                        fi
                        p_mountpoint="$val"
                    ;;

            esac
        # Кнопка "Сохранить"
        elif [ "$ret" -eq 3 ]; then
            if [ "$p_erase" = "y" ] && [ -z "$p_fs_type" ]; then
                msg="Не выбран тип файловой системы"
                unset dialog_opt dialog_extra dialog_init
                dialog_call --msgbox "$msg"
                continue
            fi
            part_fs_type[$index]="$p_fs_type"
            part_mountpoint[$index]="$p_mountpoint"
            part_erase[$index]="$p_erase"
            if [ "$p_fs_type" = "swap" ]; then
                fdisk_part_type=82
            elif [ "$p_type" = "extended" ]; then
                fdisk_part_type=5
            else
                fdisk_part_type=83
            fi
            ########
            # Строка для fdisk
            echo ${part_name[$index]} | grep -q "free" && (echo "n"
            if [ $GUID -eq 0 ]; then
                if echo "${part_name[@]}" | grep -q "^<free" || [ "${p_type::1}" != "l" ] ; then
                    # Тип раздела
                    echo "${p_type::1}"
                    if [ $(LC_ALL=C fdisk -w always -W always -l $temp_target_dev | grep -c -e 'Linux$' -e 'Linux swap') -lt 3 ]; then
                        # Номер раздела
                        echo ""
                    fi
                fi
            elif [ "${#part_name[@]} -lt 127" ]; then
                # Номер раздела
                    echo ""
            fi
            echo "${part_start[$index]}"
            echo "${p_end}"
            if [ "${part_fs_type[$index]}" = "swap" ]; then
                echo "t"
                echo ""
                echo "82"
            fi
            echo "w") | $FDISK $temp_target_dev >> $INST_LOG 2>&1
            return 0

        # кнопка "Назад"
        elif [ "$ret" -eq 1 ]; then
            return 1
        fi
    done



    ## END FS tune dialog
}

read_sfdisk_in()
{
    DEV="$1"
    if [ $GUID -eq 0 ]; then
        LC_ALL=C sfdisk -l $DEV -o device,start,end,size,id | grep "^$DEV"  > $FDISK_TEMP
    else
        LC_ALL=C sfdisk -l $DEV -o device,start,end,size | grep "^$DEV"  > $FDISK_TEMP
    fi
    LC_ALL=C sfdisk -F $DEV | sed -n '/Start/,$p' | tail +2 | while read line; do echo "free $line"; done >> $FDISK_TEMP

    unset part_n
    unset part_name
    unset part_start
    unset part_end
    unset part_size
    unset part_type
    pnum=0;

    while read line; do
        part_n[$pnum]=`echo $line | cut -d' ' -f1`
        part_n[$pnum]="${part_n[$pnum]#$DEV}"
        part_name[$pnum]=`echo $line | cut -d' ' -f1 | sed 's/\/.*\///g'`
        if [ "${part_name[$pnum]}" = "free" ]; then
            part_name[$pnum]="<free space>"
            part_size[$pnum]=`echo $line | cut -d' ' -f5`
            part_start[$pnum]=`echo $line | cut -d' ' -f2`
            part_end[$pnum]=`echo $line | cut -d' ' -f3`
            part_type[$pnum]=""
        else
            part_size[$pnum]=`echo $line | cut -d' ' -f4`
            part_start[$pnum]=`echo $line | cut -d' ' -f2`
            part_end[$pnum]=`echo $line | cut -d' ' -f3`
            part_type[$pnum]=`echo $line | cut -d' ' -f5`

            if $(is_part_logical "$pnum"); then
                part_type[$pnum]="logical"
            elif [ "${part_type[$pnum]}" = "83" ]; then
                part_type[$pnum]="primary"
            elif [ "${part_type[$pnum]}" = "5" ]; then
                part_type[$pnum]="extended"
            elif [ "${part_type[$pnum]}" = "82" ]; then
                part_type[$pnum]="swap"
            else
                part_type[$pnum]=""
            fi

            if [ "${part_type[$pnum]}" == "extended" ]
            then
                part_fs_type[$pnum]=
            elif [ "${part_erase[$pnum]}" != "y" ]
            then
                p_fs_type=$(lsblk -f ${TARGET_DEV}${part_n[$pnum]} -o FSTYPE -n 2>> /$INST_LOG)
                [ -n "$p_fs_type" ] && part_fs_type[$pnum]="$p_fs_type"
            fi
       fi
        pnum=$(($pnum+1))
    done < <(sort -h -k 2 $FDISK_TEMP | sed 's/[[:space:]]\+/ /g')
    if [ $pnum -eq 0 ]; then
        part_name[0]="<free space>"
        part_start[0]=2048
        part_end[0]=$(($(LC_ALL=C fdisk -l $DEV | grep -o "[0-9]*.sectors$" | cut -d' ' -f1)-1))
        part_size[0]=$(sectors_to_human_bytes "$((${part_end[0]}-${part_start[0]}))")
        part_type[$pnum]=""
    fi
}

dev_partition_dialog()
{
    rm -f $LSBLK_LOG
    rm -f $DIALOG_FILE
    rm -f $MSG_LOG
    local cycle_flg=0

    TITLE="Разметка диска"
    dialog_call --infobox "Считывание/запись параметров..."

    [[ "$part_resume_flg" == 0 ]] && unset_globals

    # считываем разметку из временного файла
    read_sfdisk_in "$temp_target_dev"

    if [[ "$part_resume_flg" == 0 ]]; then
        for ((pnum=0; pnum < ${#part_name[@]}; pnum = pnum + 1)); do
            part_fs_type[$pnum]="$(LC_ALL=C lsblk -n -oFSTYPE ${TARGET_DEV}${part_n[$index]} 2>> $INST_LOG)"
            echo "$dialog_fs_list" | grep -q "${part_fs_type[$pnum]}"  && part_fs_type[$pnum]=""
        done
    fi

    while true; do
        if [[ "$cycle_flg" == 1 ]] && [[ "$part_resume_flg" == 0 ]]; then
            TITLE="Разметка диска"
            dialog_call --infobox "Считывание/запись параметров..."
            read_sfdisk_in "$temp_target_dev"
        fi
        part_resume_flg=0
        cycle_flg=1

        # Формирование отображаемой пользователю таблицы разделов
        unset dialog_opt dialog_extra dialog_init
        for ((pnum=0; pnum < ${#part_name[@]}; pnum = pnum + 1)); do
            table_row=""
            if $(is_part_logical "$pnum"); then
                table_row="  ${part_name[$pnum]}|"
            else
                table_row="${part_name[$pnum]}|"
            fi
                table_row="$table_row ${part_size[$pnum]}| ${part_type[$pnum]}|"
                if [ $GUID -eq 1 ] || [ -n "${part_type[$pnum]}" ] && \
                   [ "${part_type[$pnum]}" != "extended" ] && \
                   [ "${part_type[$pnum]}" != "swap" ] ; then
                    table_row="$table_row ${part_fs_type[$pnum]}| ${part_mountpoint[$pnum]}|"
                fi
                dialog_opt+=("$pnum" "$table_row")
         done
        dialog_extra=("--column-separator" "|")
        dialog_extra+=("--help-button" "--help-label" "Назад")
        dialog_key=$(dialog_call --menu "Разметка диска $TARGET_DEV $(lsblk -d -n -o SIZE,LABEL,MODEL $TARGET_DEV 2>/dev/null)" "Выбрать" "Удалить/сброс" "Записать")
        ret=$?
        # "Выбрать" или "Удалить"
        if [ "$ret" -eq 0 ] || [ "$ret" -eq 3 ]; then
            if [ -z "$dialog_key" ]; then
                msg="Ошибка: не выбран раздел"
                unset dialog_opt dialog_extra dialog_init
                dialog_call --msgbox "$msg"
                continue
            fi
            # "Выбрать"
            if [ "$ret" -eq 0 ]; then
                if [ $GUID -eq 0 ] && echo "${part_name[$dialog_key]}" | grep -q -v "<free" && [ -z "${part_type[$dialog_key]}" ]; then
                    unset dialog_opt dialog_extra dialog_init
                    dialog_call --yesno "Неизвестный тип раздела. Удалить раздел?"
                    [[ $? -eq 0 ]] && part_delete_dialog "$dialog_key"
                    continue
                fi
                part_edit_dialog $dialog_key
            # "Удалить"
            else
                part_delete_dialog $dialog_key
            fi
        # "Записать"
        elif [ $ret -eq 1 ]; then
            # удаляем пустые (<free space>) разделы из списков
            delete_free_space
            check_root_boot
            [ $? -ne 0 ] && continue
            break
        # "Назад" (help button)
        elif [ $ret -eq 2 ]; then
            return 33
        else
            return $ret
        fi
    done

    return 0
}

fdisk_func()
{
    [[ "$target_type" != "device" ]] && return 0

    TITLE="Разметка диска"
    local param=

    if [ $format_mode -eq 0 -o $format_mode -eq 1 -o $format_mode -eq 2 ];
    then
        msg="\n                 Очистка старой разметки..."
        dialog_infobox "$msg"
        # 140688 - sfdisk и partx возвращают 1 при отсутствии разметки
        sfdisk --delete $TARGET_DEV &>> $INST_LOG
        partx  --show   $TARGET_DEV &>> $INST_LOG

        msg="\n                 Разбиение на разделы..."
        dialog_infobox "$msg"
        sfdisk -d $temp_target_dev | sfdisk $TARGET_DEV >> $INST_LOG 2>&1 || do_fatal "Ошибка разметки sfdisk"
        partx -u $TARGET_DEV >> $INST_LOG 2>&1 || do_fatal "Ошибка обновления разметки partx"

        # Обход для 80719
        # ждем создания файлов устройств после удаления udev'ом
        # fdisk ждет перечитывания таблицы разделов
        # используется тут как задержка
        fdisk -l >/dev/null 2>>$INST_LOG

        # Проверяем создания файла первой партиции
        # Таймаут 60с
        res=1
        i=0
        while [ $res -ne 0 -a $i -lt 60 ]
        do
            ls ${TARGET_DEV}${PART_PREFIX}${part_n[1]} &>/dev/null
            [ $? == 0 ] && res=0
            let i++
            sleep 1
        done

        # в лог для наглядности
        lsblk -o NAME,SIZE,UUID &>> $INST_LOG

        return 0
    fi

    echo "Ошибка: параметр \"format_mode\" не задан" >> $INST_LOG
    do_fatal
}


format_partitions()
{
    [[ "$target_type" != "device" ]] && return 0

    TITLE="Разметка диска"
    local mkfs_opts=

    percent=0
    step=$[100/${#part_name[@]}]

    #инициализация разделов
    echo '0' > $RESULT
    for ((pnum=0; pnum<${#part_name[@]}; pnum=pnum+1)); do

        if [ "${part_erase[$pnum]}" != "y" ]; then
            continue
        fi
 
        case ${part_fs_type[$pnum]} in
            ext4|ext3|ext2|xfs)
            [[ "${part_fs_type[$pnum]}" == xfs ]] && mkfs_opts="-f" || mkfs_opts="-F"
            if [[ $dialog_min == 0 ]]
            then
                (mkfs.${part_fs_type[$pnum]} ${mkfs_opts} ${TARGET_DEV}${part_n[$pnum]} >>$INST_LOG 2>&1 && echo 1 >$progress || echo '1' >$RESULT) |
                do_progress $percent "\n      Инициализация раздела ${TARGET_DEV}${part_n[$pnum]}" |
                $DIALOG $D_OPT  --backtitle "$BACKTITLE" --title "$TITLE" --gauge "\n      Инициализация раздела ${TARGET_DEV}${part_n[$pnum]}" 8 60
                [ $? -ne 0 ] && do_fatal
                [ `cat $RESULT` -ne 0 ] && do_fatal
            else
                msg="\n      Инициализация раздела ${TARGET_DEV}${part_n[$pnum]}"
                dialog_infobox "$msg"

                mkfs.${part_fs_type[$pnum]} ${mkfs_opts} ${TARGET_DEV}${part_n[$pnum]} >> $INST_LOG 2>&1
                [ $? -ne 0 ] && do_fatal
            fi

            ;;
            swap)
            if [[ $dialog_min == 0 ]]
            then
                #note: для swap ставится задержка sleep 1, т.к. при быстром срабатывании mkswap прогресс-бар может зависнуть
                #из-за того, что команда rm -f $progress будет выполнена после echo 1>$progress
                (mkswap ${TARGET_DEV}${part_n[$pnum]} >> $INST_LOG 2>&1 && sleep 1 && echo 1 > $progress || echo '1' > $RESULT) |
                do_progress $percent "\n      Инициализация раздела ${TARGET_DEV}${part_n[$pnum]}" |
                $DIALOG $D_OPT  --backtitle "$BACKTITLE" --title "$TITLE" --gauge "\n      Инициализация раздела ${TARGET_DEV}${part_n[$pnum]}" 8 60
                [ $? -ne 0 ] && do_fatal
                [ `cat $RESULT` -ne 0 ] && do_fatal
            else
                msg="\n      Инициализация раздела ${TARGET_DEV}${part_n[$pnum]}"
                dialog_infobox "$msg"

                mkswap ${TARGET_DEV}${part_n[$pnum]} >> $INST_LOG 2>&1
                [ $? -ne 0 ] && do_fatal
            fi
            ;;
            *)
                msg="Файловая система ${part_fs_type[$pnum]} не поддерживается"
                unset dialog_opt dialog_extra dialog_init
                dialog_call --msgbox "$msg"
                do_fatal
            ;;
        esac

        if [[ $dialog_min == 0 ]]
        then
            percent=$[$percent+$step]
        fi

    done


    if [[ $dialog_min == 0 ]]
    then
        (sleep 1 && echo 1 > $progress) | do_progress 100 "\n      Инициализация раздела ${TARGET_DEV}${part_n[$pnum]}" |
        $DIALOG $D_OPT  --backtitle "$BACKTITLE" --title "$TITLE" --gauge "\n      Инициализация раздела ${TARGET_DEV}${part_n[$pnum]}" 8 60
    fi

    #tune ext
    for ((pnum=0; pnum<${#part_name[@]}; pnum=pnum+1)); do
        [ "${part_name[$pnum]}" == "<free space>" ] && continue
        case ${part_fs_type[$pnum]} in
        ext4|ext3|ext2)

            echo "tune2fs -i0 -c0 ${TARGET_DEV}${part_n[$pnum]}" >> $INST_LOG
            tune2fs -i0 -c0 ${TARGET_DEV}${part_n[$pnum]} >> $INST_LOG 2>&1
            if [[ $? != 0 ]]; then
                ehco "fsck -fy ${TARGET_DEV}${part_n[$pnum]}" >> $INST_LOG
                fsck -fy ${TARGET_DEV}${part_n[$pnum]} >> $INST_LOG 2>&1
                [[ $? != 0 ]] && do_fatal
            fi
            ;;
        esac
    done
}
